CREATE TABLE IF NOT EXISTS RM_ENVIRONMENT(
  ID INTEGER NOT NULL AUTO_INCREMENT,
  NAME VARCHAR(128) NOT NULL,
  CONSTRAINT PK_RM_ENVIRONMENT_01 PRIMARY KEY (ID),
  CONSTRAINT UNQ_RM_ENVIRONMENT_01 UNIQUE (NAME)
)ENGINE INNODB;

CREATE TABLE IF NOT EXISTS RM_SERVER_INSTANCE (
  ID INTEGER NOT NULL AUTO_INCREMENT,
  ENVIRONMENT_ID INTEGER NOT NULL,
  NAME VARCHAR(128) NOT NULL,
  SERVER_URL VARCHAR(1024) NOT NULL,
  DBMS_TYPE VARCHAR(128) NOT NULL,
  INSTANCE_TYPE VARCHAR(128) NOT NULL,
  SERVER_CATEGORY VARCHAR(128) NOT NULL,
  ADMIN_USERNAME VARCHAR(128),
  ADMIN_PASSWORD VARCHAR(255),
  TENANT_ID INTEGER NOT NULL,
  DRIVER_CLASS VARCHAR(128) NOT NULL,
  SSH_HOST VARCHAR(128),
  SSH_PORT INTEGER,
  SSH_USERNAME VARCHAR(128),
  SNAPSHOT_TARGET_DIRECTORY VARCHAR(1024),
  CONSTRAINT UNQ_RM_SERVER_INSTANCE_01 UNIQUE  (NAME, ENVIRONMENT_ID, TENANT_ID),
  CONSTRAINT PK_RM_SERVER_INSTANCE_01 PRIMARY KEY (ID),
  CONSTRAINT FK_RM_SERVER_INSTANCE_01 FOREIGN KEY (ENVIRONMENT_ID) REFERENCES RM_ENVIRONMENT (ID) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE IF NOT EXISTS RM_DATABASE (
  ID INTEGER NOT NULL AUTO_INCREMENT,
  NAME VARCHAR(128) NOT NULL,
  RSS_INSTANCE_ID INTEGER NOT NULL,
  TYPE VARCHAR(15) NOT NULL,
  TENANT_ID INTEGER NOT NULL,
  CONSTRAINT UNQ_RM_DATABASE_01 UNIQUE (NAME, RSS_INSTANCE_ID, TENANT_ID),
  CONSTRAINT PK_RM_DATABASE_01 PRIMARY KEY (ID),
  CONSTRAINT FK_RM_DATABASE_01 FOREIGN KEY (RSS_INSTANCE_ID) REFERENCES RM_SERVER_INSTANCE (ID) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE IF NOT EXISTS RM_DATABASE_USER (
  ID INTEGER NOT NULL AUTO_INCREMENT,
  USERNAME VARCHAR(16) NOT NULL,
  ENVIRONMENT_ID INTEGER NOT NULL,
  RSS_INSTANCE_ID INTEGER NOT NULL DEFAULT -1,
  TYPE VARCHAR(15) NOT NULL,
  TENANT_ID INTEGER NOT NULL,
  CONSTRAINT UNQ_RM_DATABASE_USER_01 UNIQUE (USERNAME, TENANT_ID, ENVIRONMENT_ID, RSS_INSTANCE_ID),
  CONSTRAINT PK_RM_DATABASE_USER_01 PRIMARY KEY (ID)
)ENGINE INNODB;

CREATE TABLE IF NOT EXISTS RM_USER_DATABASE_ENTRY (
  ID INTEGER NOT NULL AUTO_INCREMENT,
  DATABASE_USER_ID INTEGER NOT NULL,
  DATABASE_ID INTEGER NOT NULL,
   CONSTRAINT PK_RM_USER_DATABASE_ENTRY_01 PRIMARY KEY (ID),
  CONSTRAINT UNQ_RM_USER_DATABASE_ENTRY_01 UNIQUE (DATABASE_USER_ID, DATABASE_ID),
  CONSTRAINT FK_RM_USER_DATABASE_ENTRY_01 FOREIGN KEY (DATABASE_USER_ID) REFERENCES RM_DATABASE_USER (ID) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT FK_RM_USER_DATABASE_ENTRY_02 FOREIGN KEY (DATABASE_ID) REFERENCES RM_DATABASE (ID) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE IF NOT EXISTS RM_USER_DATABASE_PRIVILEGE (
  ID INTEGER NOT NULL AUTO_INCREMENT,
  USER_DATABASE_ENTRY_ID INTEGER NOT NULL,
  SELECT_PRIV enum('N','Y') NOT NULL,
  INSERT_PRIV enum('N','Y') NOT NULL,
  UPDATE_PRIV enum('N','Y') NOT NULL,
  DELETE_PRIV enum('N','Y') NOT NULL,
  CREATE_PRIV enum('N','Y') NOT NULL,
  DROP_PRIV enum('N','Y') NOT NULL,
  GRANT_PRIV enum('N','Y') NOT NULL,
  REFERENCES_PRIV enum('N','Y') NOT NULL,
  INDEX_PRIV enum('N','Y') NOT NULL,
  ALTER_PRIV enum('N','Y') NOT NULL,
  CREATE_TMP_TABLE_PRIV enum('N','Y') NOT NULL,
  LOCK_TABLES_PRIV enum('N','Y') NOT NULL,
  CREATE_VIEW_PRIV enum('N','Y') NOT NULL,
  SHOW_VIEW_PRIV enum('N','Y') NOT NULL,
  CREATE_ROUTINE_PRIV enum('N','Y') NOT NULL,
  ALTER_ROUTINE_PRIV enum('N','Y') NOT NULL,
  EXECUTE_PRIV enum('N','Y') NOT NULL,
  EVENT_PRIV enum('N','Y') NOT NULL,
  TRIGGER_PRIV enum('N','Y') NOT NULL,
  CONSTRAINT PK_RM_USER_DATABASE_PRIVILEGE_01 PRIMARY KEY (ID),
  CONSTRAINT UNQ_RM_USER_DATABASE_PRIVILEGE_01 UNIQUE (USER_DATABASE_ENTRY_ID),
  CONSTRAINT FK_RM_USER_DATABASE_PRIVILEGE_01 FOREIGN KEY (USER_DATABASE_ENTRY_ID) REFERENCES RM_USER_DATABASE_ENTRY (ID) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE IF NOT EXISTS RM_SYSTEM_DATABASE_COUNT (
  ID INTEGER NOT NULL AUTO_INCREMENT,
  ENVIRONMENT_ID INTEGER NOT NULL,
  COUNT INTEGER NOT NULL DEFAULT 0,
  CONSTRAINT PK_RM_SYSTEM_DATABASE_COUNT_01 PRIMARY KEY (ID),
  CONSTRAINT FK_RM_SYSTEM_DATABASE_COUNT_01 FOREIGN KEY (ENVIRONMENT_ID) REFERENCES RM_ENVIRONMENT (ID)
)ENGINE INNODB;

CREATE TABLE IF NOT EXISTS RM_DB_PRIVILEGE_TEMPLATE (
  ID INTEGER NOT NULL AUTO_INCREMENT,
  ENVIRONMENT_ID INTEGER NOT NULL,
  NAME VARCHAR(128) NOT NULL,
  TENANT_ID INTEGER NOT NULL,
  CONSTRAINT PK_RM_DB_PRIVILEGE_TEMPLATE_01 PRIMARY KEY (ID),
  CONSTRAINT UNQ_RM_DB_PRIVILEGE_TEMPLATE_01 UNIQUE (ENVIRONMENT_ID, NAME, TENANT_ID),
  CONSTRAINT FK_RM_DB_PRIVILEGE_TEMPLATE_01 FOREIGN KEY (ENVIRONMENT_ID) REFERENCES RM_ENVIRONMENT (ID) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE IF NOT EXISTS RM_DB_PRIVILEGE_TEMPLATE_ENTRY (
  ID INTEGER NOT NULL AUTO_INCREMENT,
  TEMPLATE_ID INTEGER NOT NULL,
  SELECT_PRIV enum('N','Y') NOT NULL,
  INSERT_PRIV enum('N','Y') NOT NULL,
  UPDATE_PRIV enum('N','Y') NOT NULL,
  DELETE_PRIV enum('N','Y') NOT NULL,
  CREATE_PRIV enum('N','Y') NOT NULL,
  DROP_PRIV enum('N','Y') NOT NULL,
  GRANT_PRIV enum('N','Y') NOT NULL,
  REFERENCES_PRIV enum('N','Y') NOT NULL,
  INDEX_PRIV enum('N','Y') NOT NULL,
  ALTER_PRIV enum('N','Y') NOT NULL,
  CREATE_TMP_TABLE_PRIV enum('N','Y') NOT NULL,
  LOCK_TABLES_PRIV enum('N','Y') NOT NULL,
  CREATE_VIEW_PRIV enum('N','Y') NOT NULL,
  SHOW_VIEW_PRIV enum('N','Y') NOT NULL,
  CREATE_ROUTINE_PRIV enum('N','Y') NOT NULL,
  ALTER_ROUTINE_PRIV enum('N','Y') NOT NULL,
  EXECUTE_PRIV enum('N','Y') NOT NULL,
  EVENT_PRIV enum('N','Y') NOT NULL,
  TRIGGER_PRIV enum('N','Y') NOT NULL,
  CONSTRAINT PK_RM_DB_PRIVILEGE_TEMPLATE_ENTRY_01 PRIMARY KEY (ID),
  CONSTRAINT FK_RM_DB_PRIVILEGE_TEMPLATE_ENTRY_01 FOREIGN KEY (TEMPLATE_ID) REFERENCES RM_DB_PRIVILEGE_TEMPLATE (ID) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE INNODB;

CREATE TABLE IF NOT EXISTS RM_USER_INSTANCE_ENTRY (
  RSS_INSTANCE_ID INTEGER NOT NULL,
  DATABASE_USER_ID INTEGER NOT NULL,
 CONSTRAINT PK_RM_USER_INSTANCE_ENTRY_01 PRIMARY KEY (RSS_INSTANCE_ID,DATABASE_USER_ID) ,
 CONSTRAINT FK_RM_USER_INSTANCE_ENTRY_01 FOREIGN KEY (DATABASE_USER_ID) REFERENCES RM_DATABASE_USER (ID ) ON DELETE CASCADE ON UPDATE CASCADE,
 CONSTRAINT FK_RM_USER_INSTANCE_ENTRY_02 FOREIGN KEY (RSS_INSTANCE_ID ) REFERENCES RM_SERVER_INSTANCE (ID ) ON DELETE CASCADE ON UPDATE CASCADE 
)ENGINE = InnoDB;




